#include "ConversionArgs.h"
#include <iostream>
using namespace std;

void ConversionArgs::getCmdLineArgs(int argc, char **argv){
	// set default wave parameters
	nOfChannels = 1;
	samplingRate = 44100;
	bitsPerSample = 24;
	string param;

	// parse the command line
	while ((argc>1)&&(param = *++argv)[0] == '-') {
		--argc;
		switch (param[1]) {
			case ('b'):
				bitsPerSample = atoi(&param[2]);
				break;
			case ('r'):
				samplingRate = atoi(&param[2]);
				break;
			case ('c'):
				nOfChannels = atoi(&param[2]);
				break;
			case('?'):
				usage();
		}
	}

	// check for enough information
	if (argc < 2) {
		usage();
	}

	cout << "bits: <" << bitsPerSample << ">, rate: <" << samplingRate << ">, channels: <";
	cout << nOfChannels << ">" << endl;

	while(*argv != 0){
		fInFilenames.push_back(*argv);
		fOutFilenames.push_back(makeOutfilename(*argv));
		argv++;
	}

}

void ConversionArgs::usage(){
	const string HELPSTRING = "sd2wav [-r<rate>] [-b<bits per sample>] [-c<channels>] infile [infile2 ... ]";
	cout << endl << HELPSTRING << endl;
	exit(0);
}

ConversionArgs::ConversionArgs(){}

ConversionArgs::ConversionArgs(int argc, char **argv){
	getCmdLineArgs(argc, argv);
}

string ConversionArgs::makeOutfilename(string infilename){
		string outfilename = infilename;
		int pos;
		if((pos = outfilename.rfind(".")) != string::npos){
			outfilename.erase(pos);
		}
		outfilename += ".wav";
		return outfilename;
}

filenameListIterator_t ConversionArgs::getInfilelistBegin() {
	return fInFilenames.begin();
}

filenameListIterator_t ConversionArgs::getInfilelistEnd() {
	return fInFilenames.end();
}

filenameListIterator_t ConversionArgs::getOutfilelistBegin() {
	return fOutFilenames.begin();
}

filenameListIterator_t ConversionArgs::getOutfilelistEnd() {
	return fOutFilenames.end();
}