#include <iostream>
using namespace std;

#include "MsWaveWriter.h"

#include "wave.h"

MsWaveWriter::MsWaveWriter(const ConversionArgs& conversionArgs, const string& outFilename) : fnameo(outFilename) {
	totalSamples = 0;
	wfOut.wFormatTag = WAVE_FORMAT_PCM;
	wfOut.nChannels = conversionArgs.nOfChannels;
	wfOut.nSamplesPerSec = conversionArgs.samplingRate;
	wfOut.nAvgBytesPerSec = conversionArgs.nOfChannels * conversionArgs.samplingRate;
	wfOut.wBitsPerSample = conversionArgs.bitsPerSample;
	wfOut.nBlockAlign = conversionArgs.nOfChannels * conversionArgs.bitsPerSample / 8;
	wfOut.cbSize = 0;
	int ret = WaveCreateFile((char*)(fnameo.c_str()), &hOut, &wfOut, &ckOut, &ckOutRiff);
	if (ret != MMSYSERR_NOERROR) {
		cout << "Error creating <" << fnameo <<">, code <"<< ret << ">" << endl << endl;
		exit(ret);
	}
	ret = WaveStartDataWrite(&hOut, &ckOut, &mmInfoOut);
	if (ret != MMSYSERR_NOERROR) {
		cout << "Error creating <" << fnameo <<">, code <"<< ret << ">" << endl << endl;
		exit(ret);
	}
}

MsWaveWriter::~MsWaveWriter() {
	int ret = WaveCloseWriteFile(&hOut, &ckOut, &ckOutRiff, &mmInfoOut, totalSamples);
	if (ret != MMSYSERR_NOERROR) {
		cout << "Error writing to <" << fnameo <<">, code <"<< ret << ">" << endl << endl;
		exit(ret);
	}
}

// copy the buffer, doing byte order reversal
void MsWaveWriter::writeChunk(const BYTE iBuf[], size_t actualIn){
	// copy the buffer, doing byte order reversal
	int i = 0;
	UINT actualOut;
	do {
		if (wfOut.wBitsPerSample == 16) {
			oBuf [i] = iBuf [i+1];
			oBuf [i+1] = iBuf [i];
			i += 2;
		} else if (wfOut.wBitsPerSample == 24) {
			oBuf [i] = iBuf [i+2];
			oBuf [i+1] = iBuf [i+1];
			oBuf [i+2] = iBuf [i];
			i += 3;
		} else {
			cout << "bits per sample must be either 16 or 24 but were: <" 
				 << ">" << wfOut.wBitsPerSample << endl;
		}
	} while (i < BUFFERSIZE);
	int ret = WaveWriteFile(hOut, actualIn, (unsigned char*)&oBuf, &ckOut, 
		&actualOut, &mmInfoOut);
	if ((ret != MMSYSERR_NOERROR) || (actualOut != actualIn)) {
		cout << "Error writing to <" << fnameo <<">, code <"<< ret << ">" << endl << endl;
		exit(ret);
	}
	totalSamples += actualIn / wfOut.nBlockAlign;
}