#ifndef _SOUNDDESIGNER2READER_H_
#define _SOUNDDESIGNER2READER_H_
#include <string>
#include <fstream>

#include <windows.h>

#include "ConversionArgs.h" // for BUFFERSIZE

// 
// This is why we can't determine the channels, samplesPerSecond and bitsPerSample from the
// SoundDesigner2 file but rather need to specify them explicitly:
// [Kussmaul91] says:
// Sound Designer II is a Macintosh based [audio file format] which keeps header information 
// in the resource fork, which is not copied when the file is transferred, 
// so the sampling rate [, bitsPerSample, nOfChannels] must be given by the user.
//
// [Kussmaul91] Clifton Kussmaul: "APPLICATIONS OF WAVELETS IN MUSIC - The Wavelet Function Library",
//              Dartmouth College, Hanover, New Hampshire, June 1991, 
// http://www.daimi.au.dk/~pmn/spf02/CDROM/pr1/Litteratur/applications%20of%20wavelets%20in%20music.pdf

class SoundDesigner2Reader {
public:

	SoundDesigner2Reader(const string& fnamei);

	~SoundDesigner2Reader();

	size_t readChunk();
	
	const BYTE* getBuf() const;

	bool isOk() const;


private:
	std::string fnamei;
	std::ifstream fsin;
	BYTE iBuf [BUFFERSIZE];
	bool fOk;
};

#endif