#include <iostream>
using namespace std;

#include "SoundDesigner2Reader.h"
#include "MsWaveWriter.h"
#include "ConversionArgs.h"

/**
 * This code is a refactoring of the one found here:
 * David McGriffy's Audio Files http://www.io.com/%7Emcgriffy/audio/
 */

int main(int argc, char **argv) {

	try {
		size_t actualIn;

		ConversionArgs conversionArgs(argc, argv);

		filenameListIterator_t init = conversionArgs.getInfilelistBegin();
		filenameListIterator_t outit = conversionArgs.getOutfilelistBegin();
		for(;init != conversionArgs.getInfilelistEnd(); ++init, ++outit){
			cout << *init << " ->  " << *outit << endl;
			SoundDesigner2Reader sd2(*init);
			if(!sd2.isOk()) continue;

			MsWaveWriter msWaveWriter(conversionArgs, *outit);
			do {

				actualIn = sd2.readChunk();
				msWaveWriter.writeChunk(sd2.getBuf(), actualIn);
			} while (actualIn == BUFFERSIZE);
		}

	} catch (ios_base::failure& f) {
		cout << "failed: " << f.what() << endl;
		return 1;
	}

	return 0;
}

/*
TODO:
The MsWaveWriter now does byte reordering. But for generalizing this conversion into
a pluggable architecture, we need to allow the insertion of strategies/decorators into
the copying loop. Objectives of the framework:
  - support a stream oriented approach, thus not forcing the loading of the entire
    source audio file before starting the conversion and writing out to the destination
	audio file.
  - Corollary: avoid to have an intermediate file format. The intermediate file format
    could be an approach to reduce the combinatorial explosion from n x m to n + m but
	would imply to convert a file in 2 steps:
	1. convert source file into generic format
	2. convert generic format to destination format.
  - To reduce the n x m converter overhead find a pluggable/declarative approach, e.g.:
    sd2 -> wav  requires swapping of bytes, depending on 16 or 24 bits/sample
	wav -> aiff requires ...
	sd2 -> aiff requires ...
  - The strategies/decorators will be chosen according to the parameters in ConversionArgs.
  - remove dependency on MS code, use E:\projects\met\workspace\met\met_src\AUDIO\WavePort.C
    instead
  - MediaPlayer can't read the wav files generated! Why? Another reason to switch to WavePort.C
  - introduce Exception Hierarchy
  - but before hacking on too far first check:
    http://ffmpeg.sourceforge.net/ffmpeg-doc.html#SEC26
	  great functionality but oh no, ANSI C!
	http://sourceforge.net/projects/qtcomponents/
	http://sourceforge.net/projects/sox/
	  great functionality but oh no, ANSI C!
	http://www.zip.com.au/~erikd/libsndfile/
	  great functionality but oh no, ANSI C!
	  could be used from my library, though
    http://tcvp.sourceforge.net/

	or, for that matter:

	http://sourceforge.net/softwaremap/trove_list.php?form_cat=119
*/