/*==========================================================================
 *
 *  Copyright (C) 1995-1996 Microsoft Corporation. All Rights Reserved.
 *
 *  File:		wave.h
 *  Content:	wave header
 *
 ***************************************************************************/
#ifndef __WAVE_H__
#define __WAVE_H__
#include <windows.h>

#ifdef __cplusplus
extern "C" {
#endif

#define WAVEVERSION 1

#ifndef ER_MEM
#define ER_MEM 				0xe000
#endif

#ifndef ER_CANNOTOPEN
#define ER_CANNOTOPEN 		0xe100
#endif

#ifndef ER_NOTWAVEFILE
#define ER_NOTWAVEFILE 		0xe101
#endif

#ifndef ER_CANNOTREAD
#define ER_CANNOTREAD 		0xe102
#endif

#ifndef ER_CORRUPTWAVEFILE
#define ER_CORRUPTWAVEFILE	0xe103
#endif

#ifndef ER_CANNOTWRITE
#define ER_CANNOTWRITE		0xe104
#endif



int WaveOpenFile(TCHAR*, HMMIO *, WAVEFORMATEX **, MMCKINFO *);
int WaveStartDataRead(HMMIO *, MMCKINFO *, MMCKINFO *);
int WaveReadFile(HMMIO, UINT, BYTE *, MMCKINFO *, UINT *);
int WaveCloseReadFile(HMMIO *, WAVEFORMATEX **);

int WaveCreateFile(TCHAR*, HMMIO *, WAVEFORMATEX *, MMCKINFO *, MMCKINFO *);
int WaveStartDataWrite(HMMIO *, MMCKINFO *, MMIOINFO *);
int WaveWriteFile(HMMIO, UINT, BYTE *, MMCKINFO *, UINT *, MMIOINFO *);
int WaveCloseWriteFile(HMMIO *, MMCKINFO *, MMCKINFO *, MMIOINFO *, DWORD);

int WaveLoadFile(TCHAR*, UINT *, WAVEFORMATEX **, BYTE **);
int WaveSaveFile(TCHAR*, UINT, DWORD, WAVEFORMATEX *, BYTE *);

int WaveCopyUselessChunks(HMMIO *, MMCKINFO *, MMCKINFO *, HMMIO *, MMCKINFO *, MMCKINFO *);
BOOL riffCopyChunk(HMMIO, HMMIO, const LPMMCKINFO);

//--------------------------------------
// start of dave's code

#define BUFSIZE (8192)
#define KSMPS (BUFSIZE/4)

// realtime output routines
int WaveOpenOutput();
int WaveCloseOutput();
int WavePlayBuffer(BYTE *, DWORD, void (*proc)());
void CALLBACK WaveOutProc( HWAVEOUT hwo, UINT uMsg, DWORD dwInstance, 
						  DWORD dwParam1, DWORD dwParam2);
int WaveResetOutput();

// file writing routines
int WaveOpenOutFile(TCHAR *filename, long samppersec, short bits, short channels);
int WaveWriteBuffer(BYTE *buf, long nBytes);
int WaveCloseOutFile();

typedef struct
{
	double fc1, fc2; // bottom and top of transition band
	int nFilt; // number of coeficients
	double *h; // the coeficients themselves
	float *x; // past nFilt inputs
	int nextX; // where the next input will go
	float *in, *out; // pointer to buffer
} FIR;

void firset(FIR *p);
void fir(FIR *p);

#ifdef __cplusplus
}
#endif


#endif